@@ -26,27 +26,31 @@ FlatButton::FlatButton(QWidget *parent)
     : QPushButton(parent)
 {
     setFlat(true);
-    setStyleSheet("text-align:left; color:black");
+    static const QString defaultStyle = QStringLiteral("text-align:left; color:black");
+    setStyleSheet(defaultStyle);
 }
 
 FlatButton::FlatButton(const QString &name, QWidget *parent)
     : QPushButton(name, parent)
 {
     setFlat(true);
-    setStyleSheet("text-align:left");
+    static const QString namedStyle = QStringLiteral("text-align:left");
+    setStyleSheet(namedStyle);
 }
 
 void FlatButton::leaveEvent(QEvent *e)
 {
     // setFlat(true);
-    setStyleSheet("text-align:left; text-decoration:none");
+    static const QString leaveStyle = QStringLiteral("text-align:left; text-decoration:none");
+    setStyleSheet(leaveStyle);
     QPushButton::leaveEvent(e);
 }
 
 void FlatButton::enterEvent(QEnterEvent *e)
 {
     // setFlat(false);
-    setStyleSheet("QPushButton { text-align:left; text-decoration:underline}; QToolTip { text-decoration: none; }");
+    static const QString enterStyle = QStringLiteral("QPushButton { text-align:left; text-decoration:underline}; QToolTip { text-decoration: none; }");
+    setStyleSheet(enterStyle);
     QPushButton::enterEvent(e);
 }
 
