@@ -23,6 +23,8 @@
 #include <QDebug>
 #include <QDir>
 #include <QDirIterator>
+#include <QFontMetrics>
+#include <QHash>
 #include <QProcess>
 #include <QRegularExpression>
 #include <QResizeEvent>
@@ -33,7 +35,10 @@
 #include "flatbutton.h"
 #include "mainwindow.h"
 #include "ui_mainwindow.h"
-#include "version.h"
+
+#ifndef VERSION
+    #define VERSION "?.?.?.?"
+#endif
 
 MainWindow::MainWindow(QWidget *parent)
     : QDialog(parent),
@@ -117,9 +122,9 @@ void MainWindow::filterDesktopEnvironmentItems()
     const QMap<QString, QString> desktopTerms {
         {"XFCE", "OnlyShowIn=XFCE"}, {"Fluxbox", "OnlyShowIn=FLUXBOX"}, {"KDE", "OnlyShowIn=KDE"}};
 
-    for (auto it = desktopTerms.keyValueBegin(); it != desktopTerms.keyValueEnd(); ++it) {
-        if (qgetenv("XDG_CURRENT_DESKTOP") != it->first) {
-            termsToRemove << it->second;
+    for (const auto &[desktop, term] : desktopTerms.asKeyValueRange()) {
+        if (qgetenv("XDG_CURRENT_DESKTOP") != desktop) {
+            termsToRemove << term;
         }
     }
     for (auto it = categories.begin(); it != categories.end(); ++it) {
@@ -155,10 +160,9 @@ QStringList MainWindow::listDesktopFiles(const QString &searchString, const QStr
     while (it.hasNext()) {
         const QString filePath = it.next();
         QFile file(filePath);
-        if (file.open(QIODevice::ReadOnly | QIODevice::Text)) {
-            QTextStream in(&file);
-            QString content = in.readAll();
-            if (content.contains(searchString)) {
+        if (file.open(QIODevice::ReadOnly)) {
+            const QByteArray content = file.readAll();
+            if (content.contains(searchString.toUtf8())) {
                 matchingFiles << filePath;
             }
         }
@@ -174,16 +178,25 @@ int MainWindow::calculateMaxElements(const QMultiMap<QString, QMultiMap<QString,
         max_elements = std::max(max_elements, static_cast<int>(categoryMap.size()));
     }
 
-    // Calculate maximum button width based on first category
-    const QString firstCategory = info_map.isEmpty() ? QString() : info_map.firstKey();
-    int max_button_width = 20;
-    for (const auto &fileInfo : info_map.value(firstCategory)) {
-        const QString &name = fileInfo.at(Info::Name);
-        max_button_width = qMax(name.size() * QApplication::font().pointSize() + icon_size, max_button_width);
+    // Only recalculate button width if not cached (expensive operation)
+    if (cached_max_button_width == 0) {
+        const QFontMetrics fm(QApplication::font());
+        constexpr int buttonPadding = 16; // Left/right padding inside button
+
+        // Check ALL categories for the widest button text
+        for (const auto &categoryMap : info_map) {
+            for (const auto &fileInfo : categoryMap) {
+                const QString &name = fileInfo.at(Info::Name);
+                const int textWidth = fm.horizontalAdvance(name);
+                cached_max_button_width = std::max(cached_max_button_width, textWidth);
+            }
+        }
+        // Add icon size, spacing between icon and text, and button padding
+        cached_max_button_width += icon_size + 8 + buttonPadding;
     }
 
     // Calculate maximum columns that fit in window width
-    return std::max(1, width() / max_button_width);
+    return std::max(1, width() / cached_max_button_width);
 }
 
 // Load info (name, comment, exec, iconName, category, terminal) to the info_map
@@ -243,10 +256,19 @@ void MainWindow::readInfo(const QMultiMap<QString, QStringList> &category_map)
 QString MainWindow::getTranslation(const QString &text, const QString &key, const QString &langRegion,
                                    const QString &lang)
 {
+    static QHash<QString, QRegularExpression> regexCache;
+
     // First try to find translation for specific region (e.g., en_US)
-    QRegularExpression re(QStringLiteral("^") + key + QStringLiteral("\\[") + langRegion + QStringLiteral("\\]=(.*)$"));
-    re.setPatternOptions(QRegularExpression::MultilineOption);
-    QRegularExpressionMatch match = re.match(text);
+    const QString regionPattern = key + QStringLiteral("[") + langRegion + QStringLiteral("]");
+    auto it = regexCache.find(regionPattern);
+    if (it == regexCache.end()) {
+        QString pattern = QStringLiteral("^") + key + QStringLiteral("\\[") + langRegion + QStringLiteral("\\]=(.*)$");
+        QRegularExpression re(pattern);
+        re.setPatternOptions(QRegularExpression::MultilineOption);
+        it = regexCache.insert(regionPattern, re);
+    }
+
+    QRegularExpressionMatch match = it->match(text);
     if (match.hasMatch()) {
         QString translation = match.captured(1).trimmed();
         if (!translation.isEmpty()) {
@@ -255,17 +277,32 @@ QString MainWindow::getTranslation(const QString &text, const QString &key, cons
     }
 
     // Fall back to general language (e.g., en)
-    re.setPattern(QStringLiteral("^") + key + QStringLiteral("\\[") + lang + QStringLiteral("\\]=(.*)$"));
-    match = re.match(text);
+    const QString langPattern = key + QStringLiteral("[") + lang + QStringLiteral("]");
+    it = regexCache.find(langPattern);
+    if (it == regexCache.end()) {
+        QString pattern = QStringLiteral("^") + key + QStringLiteral("\\[") + lang + QStringLiteral("\\]=(.*)$");
+        QRegularExpression re(pattern);
+        re.setPatternOptions(QRegularExpression::MultilineOption);
+        it = regexCache.insert(langPattern, re);
+    }
+
+    match = it->match(text);
     return match.hasMatch() ? match.captured(1).trimmed() : QString();
 }
 
 QString MainWindow::getValueFromText(const QString &text, const QString &key)
 {
-    QString pattern = QStringLiteral("^") + key + QStringLiteral("=(.*)$");
-    QRegularExpression re(pattern);
-    re.setPatternOptions(QRegularExpression::MultilineOption);
-    return re.match(text).captured(1).trimmed();
+    static QHash<QString, QRegularExpression> regexCache;
+
+    auto it = regexCache.find(key);
+    if (it == regexCache.end()) {
+        QString pattern = QStringLiteral("^") + key + QStringLiteral("=(.*)$");
+        QRegularExpression re(pattern);
+        re.setPatternOptions(QRegularExpression::MultilineOption);
+        it = regexCache.insert(key, re);
+    }
+
+    return it->match(text).captured(1).trimmed();
 }
 
 // Read the info_map and add the buttons to the UI
@@ -275,6 +312,7 @@ void MainWindow::addButtons(const QMultiMap<QString, QMultiMap<QString, QStringL
 
     const int max_columns = calculateMaxElements(info_map);
     int row = 0;
+    int actualMaxCol = 0;
 
     // Add buttons for each category
     for (auto it = info_map.cbegin(); it != info_map.cend(); ++it) {
@@ -294,9 +332,9 @@ void MainWindow::addButtons(const QMultiMap<QString, QMultiMap<QString, QStringL
         // Add buttons for this category
         int col = 0;
         for (const auto &fileInfo : categoryMap) {
-            col_count = std::max(col_count, col + 1);
             auto *btn = createButton(fileInfo);
             ui->gridLayout_btn->addWidget(btn, row, col);
+            actualMaxCol = std::max(actualMaxCol, col + 1);
 
             // Move to the next row if more items than max columns
             if (++col >= max_columns) {
@@ -305,6 +343,8 @@ void MainWindow::addButtons(const QMultiMap<QString, QMultiMap<QString, QStringL
             }
         }
     }
+
+    col_count = actualMaxCol;
     ui->gridLayout_btn->setRowStretch(row, 1);
 }
 
@@ -348,6 +388,7 @@ FlatButton *MainWindow::createButton(const QStringList &fileInfo)
 
 QIcon MainWindow::findIcon(const QString &iconName)
 {
+    static QHash<QString, QIcon> iconCache;
     static QIcon defaultIcon;
     static bool defaultIconLoaded = false;
 
@@ -359,9 +400,17 @@ QIcon MainWindow::findIcon(const QString &iconName)
         return defaultIcon;
     }
 
+    // Check cache first
+    auto cacheIt = iconCache.find(iconName);
+    if (cacheIt != iconCache.end()) {
+        return *cacheIt;
+    }
+
     // Check if the icon name is an absolute path and exists
     if (QFileInfo(iconName).isAbsolute() && QFile::exists(iconName)) {
-        return QIcon(iconName);
+        QIcon icon(iconName);
+        iconCache.insert(iconName, icon);
+        return icon;
     }
 
     // Prepare regular expression to strip extension
@@ -371,7 +420,9 @@ QIcon MainWindow::findIcon(const QString &iconName)
 
     // Return the themed icon if available
     if (QIcon::hasThemeIcon(nameNoExt)) {
-        return QIcon::fromTheme(nameNoExt);
+        QIcon icon = QIcon::fromTheme(nameNoExt);
+        iconCache.insert(iconName, icon);
+        return icon;
     }
 
     // Define common search paths for icons
@@ -384,10 +435,13 @@ QIcon MainWindow::findIcon(const QString &iconName)
                              QStringLiteral("/usr/share/icons/Adwaita/48x48/legacy/")};
 
     // Optimization: search first for the full iconName with the specified extension
-    auto it = std::find_if(searchPaths.cbegin(), searchPaths.cend(),
-                           [&](const QString &path) { return QFile::exists(path + iconName); });
+    auto it = std::ranges::find_if(searchPaths, [&](const QString &path) {
+        return QFile::exists(path + iconName);
+    });
     if (it != searchPaths.cend()) {
-        return QIcon(*it + iconName);
+        QIcon icon(*it + iconName);
+        iconCache.insert(iconName, icon);
+        return icon;
     }
 
     // Search for the icon without extension in the specified paths
@@ -398,7 +452,9 @@ QIcon MainWindow::findIcon(const QString &iconName)
         for (const auto &ext : {QStringLiteral(".png"), QStringLiteral(".svg"), QStringLiteral(".xpm")}) {
             const QString file = path + nameNoExt + ext;
             if (QFile::exists(file)) {
-                return QIcon(file);
+                QIcon icon(file);
+                iconCache.insert(iconName, icon);
+                return icon;
             }
         }
     }
@@ -409,11 +465,14 @@ QIcon MainWindow::findIcon(const QString &iconName)
             defaultIcon = QIcon();
             defaultIconLoaded = true;
         }
+        iconCache.insert(iconName, defaultIcon);
         return defaultIcon;
     }
 
     // If the icon is not "utilities-terminal", try to load the default icon
-    return findIcon(QStringLiteral("utilities-terminal"));
+    QIcon fallbackIcon = findIcon(QStringLiteral("utilities-terminal"));
+    iconCache.insert(iconName, fallbackIcon);
+    return fallbackIcon;
 }
 
 void MainWindow::btn_clicked()
@@ -451,16 +510,22 @@ void MainWindow::resizeEvent(QResizeEvent *event)
     if (event->oldSize().width() == event->size().width()) {
         return;
     }
-    const int newCount = width() / 200;
-    if (newCount == col_count || (newCount > max_elements && col_count == max_elements)) {
+
+    // Fast column calculation using cached button width (avoids full recalculation)
+    const int effectiveWidth = cached_max_button_width > 0 ? cached_max_button_width : 200;
+    const int newColCount = std::max(1, width() / effectiveWidth);
+
+    // Early exit: column count unchanged or already at max elements
+    if (newColCount == col_count || (newColCount >= max_elements && col_count == max_elements)) {
         return;
     }
-    // Finer check
-    const int max = calculateMaxElements(info_map);
-    if (col_count == max) {
+
+    // Full recalculation to get exact column count (updates max_elements too)
+    const int exactColCount = calculateMaxElements(info_map);
+    if (col_count == exactColCount) {
         return;
     }
-    col_count = max;
+    col_count = exactColCount;
 
     if (ui->textSearch->text().isEmpty()) {
         addButtons(info_map);
@@ -597,10 +662,9 @@ void MainWindow::removeEnvExclusive(QStringList *list, const QStringList &termsT
             QString fileContent = QString::fromUtf8(file.readAll());
             file.close();
 
-            bool containsTerm
-                = std::any_of(termsToRemove.cbegin(), termsToRemove.cend(), [&fileContent](const QString &term) {
-                      return fileContent.contains(term, Qt::CaseInsensitive);
-                  });
+            bool containsTerm = std::ranges::any_of(termsToRemove, [&fileContent](const QString &term) {
+                return fileContent.contains(term, Qt::CaseInsensitive);
+            });
 
             containsTerm ? it = list->erase(it) : ++it;
         } else {
